let data;

const xLabel = "Year";
const yLabel = "Starting Colonies";

const minBubbleR = 3;
const maxBubbleR = 20;

const colors = ["rgb(67, 171, 240)", "rgb(214, 50, 48)", "rgb(186, 48, 201)", "rgb(46, 228, 107)", "rgb(252, 187, 37)"];

const fetchCSV = async() => {
    //fetch csv data
    const pathToCSV = "Bee-Colony_2019-2021.csv";
    
    data = await d3.csv(pathToCSV, d => {
        return {
            state: d["State"],
            startingAmt: parseInt(d[yLabel]),
            percentLoss: parseFloat(d["Percent Loss"]) / 100, //convert 1 to 0.01
            year: d[xLabel]
        };
    });
    
}

const drawChart = async() => {

    //load in csv
    try {
        await fetchCSV();
    } catch (e) {
        console.log("there was an error loading the csv file:\n", e);
    }

    console.log([...colors]);
    console.log([...Array(5).keys()]);
    

    let canvasWidth = 800;
    let canvasHeight = 500;
    let xMargin = 200;
    let yMargin = 150;
    let w = canvasWidth - xMargin;
    let h = canvasHeight - yMargin;

    let svg = d3.select("body").append("svg")
        .attr("width", canvasWidth)
        .attr("height", canvasHeight);

    //------axis scales-------------
    //x scale: year
    let xScale = d3.scaleBand()
        .domain(data.map(d => d.year).sort())
        .range([0, w]);
    let xAxis = d3.axisBottom(xScale);

    //y scale: starting colonies
    let yScale = d3.scaleLinear()
        .domain([d3.min(data.map(d => d.startingAmt)), d3.max(data.map(d => d.startingAmt))])
        .range([h, 0]);
    let yAxis = d3.axisLeft(yScale)
        .tickFormat(d => (d/1000000).toFixed(1) + "M");

    //color scale: state
    let colorScale = d3.scaleOrdinal()
        .domain(data.map(d => d.state))
        .range([...colors]);
    
    let colorLegend = d3.legendColor()
        .scale(colorScale)
        .orient("vertical");

    //size scale: percent loss
    let sizeScale = d3.scaleLinear()
        .domain([d3.min(data.map(d => d.percentLoss)), d3.max(data.map(d => d.percentLoss))])
        .range([minBubbleR, maxBubbleR]);

    let sizeLegend = d3.legendSize()
        .scale(sizeScale)
        .shape('circle')
        .shapePadding(15)
        .orient("vertical")
        .title("Percent Loss")
        .titleWidth(100)
        .labelFormat(d3.format(".0%"));


    //------Title-----------
    svg.append("text")
        .attr("text-anchor", "middle")
        .attr("x", canvasWidth / 2)
        .attr("y", 30)
        .text("Bee Colony loss from 2019 to 2021.");

    //label x
    svg.append("text")
        .attr("class", "x label")
        .attr("text-anchor", "middle")
        .attr("x", canvasWidth / 2)
        .attr("y", canvasHeight - 15)
        .text(xLabel)

    //label y
    svg.append("text")
        .attr("class", "y label")
        .attr("transform", "rotate(-90)")
        .attr("text-anchor", "middle")
        .attr("x", canvasHeight / -2)
        .attr("y", 22)
        .text(yLabel)

    //draw color legend
    svg.append("g")
        .attr("class", "color-legend")
        .attr("transform", `translate(${canvasWidth - 120}, 20)`)
        .call(colorLegend);

    //draw size legend
    svg.append("g")
        .attr("class", "size-legend")
        .attr("transform", `translate(${canvasWidth - 110}, ${canvasHeight - 270})`)
        .call(sizeLegend);


    //prepare plot container
    let container_g = svg.append("g")
        .attr("transform", `translate(${xMargin * 2 / 5}, ${yMargin / 2})`);

    //draw x and y axis
    container_g.append("g")
        .attr("transform", `translate(0, ${h})`)
        .call(xAxis);
    container_g.append("g")
        .call(yAxis);


    //draw dots
    container_g.selectAll(".bubble")
        .data(data)
        .enter()
        .append("circle")
        .attr("transform", `translate(${w / 8 / 2},0)`) //1/2 of total width / number of time ticks
        .attr("class", "bubble")
        .attr("fill", d => colorScale(d.state))
        .attr("r", d => sizeScale(d.percentLoss))
        .attr("cx", d => xScale(d.year))
        .attr("cy", d => h - yScale(d.startingAmt));

}

drawChart();
